package cc.fascinated.bat.features.namehistory.profile.guild;

import cc.fascinated.bat.common.Serializable;
import cc.fascinated.bat.features.namehistory.NameHistoryFeature;
import cc.fascinated.bat.features.namehistory.TrackedName;
import cc.fascinated.bat.model.BatUser;
import com.google.gson.Gson;
import lombok.NoArgsConstructor;
import org.bson.Document;

import java.util.*;

/**
 * @author Fascinated (fascinated7)
 */
@NoArgsConstructor
public class NameHistoryProfile extends Serializable {
    /**
     * The name history of the user
     */
    private Map<String, List<TrackedName>> nameHistory;

    /**
     * Gets the name history of the user
     *
     * @param user the user to get the name history of
     * @return the name history of the user
     */
    public List<TrackedName> getNameHistory(BatUser user) {
        if (this.nameHistory == null) {
            this.nameHistory = new HashMap<>();
        }
        if (!this.nameHistory.containsKey(user.getId())) {
            this.nameHistory.put(user.getId(), new LinkedList<>());
        }
        return this.nameHistory.get(user.getId());
    }

    /**
     * Gets the name history of the user sorted
     *
     * @param user the user to get the name history of
     * @return the name history of the user sorted
     */
    public List<TrackedName> getNameHistorySorted(BatUser user) {
        return getNameHistory(user).stream().sorted((o1, o2) -> o2.getChangedDate().compareTo(o1.getChangedDate())).toList();
    }

    /**
     * Adds a name to the name history
     *
     * @param user the user to add the name to
     * @param name the name to add
     */
    public void addName(BatUser user, String name) {
        TrackedName trackedName = new TrackedName();
        trackedName.setName(name);
        trackedName.setChangedDate(new Date());
        getNameHistory(user).add(trackedName);
        cleanup();
    }

    /**
     * Cleans up the name history
     * <p>
     * This will remove any names that are not within
     * the size limit of the name history
     * </p>
     */
    private void cleanup() {
        for (String userId : this.nameHistory.keySet()) {
            List<TrackedName> trackedNames = this.nameHistory.get(userId);
            if (trackedNames.size() > NameHistoryFeature.NAME_HISTORY_SIZE) {
                trackedNames.sort((o1, o2) -> o2.getChangedDate().compareTo(o1.getChangedDate()));
                trackedNames.subList(NameHistoryFeature.NAME_HISTORY_SIZE, trackedNames.size()).clear();
            }
        }
    }

    @Override
    public void reset() {
        this.nameHistory = null;
    }

    @Override
    public void load(Document document, Gson gson) {
        this.nameHistory = new HashMap<>();
        for (String key : document.keySet()) {
            List<TrackedName> trackedNames = new LinkedList<>();
            for (Document trackedNameDocument : document.getList(key, Document.class)) {
                TrackedName trackedName = new TrackedName();
                trackedName.load(trackedNameDocument, gson);
                trackedNames.add(trackedName);
            }
            this.nameHistory.put(key, trackedNames);
        }
    }

    @Override
    public Document serialize(Gson gson) {
        Document document = new Document();
        for (String key : this.nameHistory.keySet()) {
            List<Document> trackedNames = new LinkedList<>();
            for (TrackedName trackedName : this.nameHistory.get(key)) {
                trackedNames.add(trackedName.serialize(gson));
            }
            document.put(key, trackedNames);
        }
        return document;
    }
}
