package cc.fascinated.bat.features.scoresaber.command.scoresaber;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.Category;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.Colors;
import cc.fascinated.bat.common.DateUtils;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.NumberFormatter;
import cc.fascinated.bat.exception.RateLimitException;
import cc.fascinated.bat.features.scoresaber.profile.user.ScoreSaberProfile;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberAccountToken;
import cc.fascinated.bat.service.ScoreSaberService;
import lombok.NonNull;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(name = "scoresaber", description = "General ScoreSaber commands", userInstall = true, category = Category.BEAT_SABER)
public class ScoreSaberCommand extends BatCommand {
    @Autowired
    public ScoreSaberCommand(@NonNull ApplicationContext context) {
        super.addSubCommands(
                context.getBean(LinkSubCommand.class),
                context.getBean(UserSubCommand.class),
                context.getBean(MeSubCommand.class),
                context.getBean(ResetSubCommand.class)
        );
    }

    /**
     * Builds the profile embed for the ScoreSaber profile
     *
     * @param user              The user to build the profile embed for
     * @param scoreSaberService The ScoreSaber service
     * @param interaction       The interaction
     */
    public static void sendProfileEmbed(boolean isSelf, BatUser user, ScoreSaberService scoreSaberService, SlashCommandInteraction interaction) {
        ScoreSaberProfile profile = user.getScoreSaberProfile();
        if (profile.getAccountId() == null) {
            if (!isSelf) {
                interaction.replyEmbeds(EmbedUtils.errorEmbed()
                                .setDescription("%s does not have a linked ScoreSaber account".formatted(user.getDiscordUser().getAsMention()))
                                .build())
                        .setEphemeral(true)
                        .queue();
            }
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("You do not have a linked ScoreSaber account")
                            .build())
                    .setEphemeral(true)
                    .queue();
            return;
        }

        try {
            long before = System.currentTimeMillis();
            boolean cached = scoreSaberService.isCached(profile.getAccountId());
            ScoreSaberAccountToken account = scoreSaberService.getAccount(profile.getAccountId());
            if (account == null) {
                if (!isSelf) {
                    interaction.replyEmbeds(EmbedUtils.errorEmbed()
                                    .setDescription("%s has an invalid ScoreSaber account linked, please ask them to re-link their account"
                                            .formatted(user.getDiscordUser().getAsMention()))
                                    .build())
                            .setEphemeral(true)
                            .queue();
                }
                interaction.replyEmbeds(EmbedUtils.errorEmbed()
                                .setDescription("You have an invalid ScoreSaber account linked, please re-link your account")
                                .build())
                        .setEphemeral(true)
                        .queue();
                return;
            }

            long fetchTime = System.currentTimeMillis() - before;
            interaction.replyEmbeds(new EmbedBuilder()
                    .setAuthor(account.getName() + "'s Profile", "https://scoresaber.com/u/%s".formatted(account.getId()),
                            "https://cdn.scoresaber.com/avatars/%s.jpg".formatted(account.getId()))
                    .addField("Name", account.getName(), true)
                    .addField("Country", account.getCountry(), true)
                    .addField("Rank", "#" + NumberFormatter.formatCommas(account.getRank()), true)
                    .addField("Country Rank", "#" + NumberFormatter.formatCommas(account.getCountryRank()), true)
                    .addField("PP", NumberFormatter.formatCommas(account.getPp()), true)
                    .addField("Joined", "<t:%s>".formatted(DateUtils.getDateFromString(account.getFirstSeen()).toInstant().getEpochSecond()), true)
                    .setTimestamp(LocalDateTime.now())
                    .setFooter(!cached ? "Fetched in %sms".formatted(fetchTime) : "Cached", "https://flagcdn.com/h120/%s.png".formatted(account.getCountry().toLowerCase()))
                    .setColor(Colors.DEFAULT)
                    .build()).queue();
        } catch (RateLimitException ex) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("The ScoreSaber API is currently rate our limiting requests, please try again later")
                            .build())
                    .setEphemeral(true)
                    .queue();
        }
    }
}
